from .base import BaseModel
from typing import Optional


class MockModel(BaseModel):

    def complete(
        self,
        query: str,
        context: str,
        task_description: Optional[str] = None,
        system_prompt: Optional[str] = None,
    ):
        return context.replace("\n", "")

    def model_type(self):
        return "text"



from ..utils import most_common
from ..models import BaseModel
from .base import BaseExtractor
from typing import List, Optional
import re


class CategoryExtractor(BaseExtractor):
    def __init__(
        self,
        name: str,
        query: str,
        categories: List[str],
        model: BaseModel,
        judge_model: Optional[BaseModel] = None,
        max_chunk_size: float = 10_000,
    ) -> None:
        super().__init__(
            name=name, query=query, max_chunk_size=max_chunk_size, model=model
        )
        self.categories = categories

    def extract(self, doc_text: str) -> str:
        merged_chunks = self._chunk_text(doc_text)
        categories_str = "The possible categories are " + ", ".join(
            [f'"{w}"' for w in self.categories]
        )
        snip_messages = []

        # Classifier models don't have actual queries, you just provide the possible categories.
        if self.model.model_type() == "classifier":
            query = self.categories
        else:
            query = self.query

        for snippet in merged_chunks:

            message = self.model.complete(
                query=query,
                context=snippet,
                task_description=f"Valid categories are: {categories_str} \n Use the information below:",
                system_prompt='Your job is to provide a categorial answer based on provided text. Answer only with the category, and no other text. If there is no relevant information in the text provided, respond with "NA". Do not make things up.',
            )
            snip_messages.append(message)
            print("Snippet:" + snippet)
            print("Message" + message)

        valid_answers = [
            m for m in snip_messages if (m != "NA") and (m in self.categories)
        ]

        if len(valid_answers) > 0:
            consensus = most_common(valid_answers)
        else:
            consensus = "NA"

        return consensus



from .extractors import (
    BaseExtractor,
    NumericExtractor,
    CategoryExtractor,
    TextExtractor,
)
from .nodes import Node

from .models import BaseModel, AnthropicAPIModel, MockModel, TransformersQAModel



from .base import BaseModel
from transformers import pipeline
from typing import Optional


class TransformersQAModel(BaseModel):

    def __init__(
        self,
        model: str,
        na_threshold: float = 0.5,
        temperature: float = 0,
        max_tokens: int = 1000,
    ) -> None:
        super().__init__(model, temperature, max_tokens)
        self.pipeline = pipeline("question-answering", model=model, tokenizer=model)
        self.na_threshold = na_threshold

    def model_type(self):
        return "text"

    def complete(
        self,
        query: str,
        context: str,
        task_description: Optional[str] = None,
        system_prompt: Optional[str] = None,
    ):
        input = {"question": query, "context": context}

        res = self.pipeline(input)
        print(res)
        if res["score"] >= self.na_threshold:
            return res["answer"]
        else:
            return "NA"



from ..utils import parseNumber, most_common
from ..models import BaseModel
from .base import BaseExtractor
from typing import List, Optional
import re


class NumericExtractor(BaseExtractor):

    def extract(self, doc_text: str) -> float:

        merged_chunks = self._chunk_text(doc_text)
        snip_messages = []
        for snippet in merged_chunks:

            message = self.model.complete(
                query=self.query,
                context=snippet,
                task_description="Use the information given below.",
                system_prompt='Your job is to extract a numerical value from a document. Respond with a single number. Do not explain your answer. Do not provide context. If there is no relevant information in the text provided, respond with "NA". Do not make things up.',
            )
            print(message)
            snip_messages.append(message)

        valid_answers = [m for m in snip_messages if m != "NA"]

        numerized = []
        for answer in valid_answers:

            num = "".join(
                re.findall(
                    "[-+]?[.]?[\d]+(?:,\d\d\d)*[\.]?\d*(?:[eE][-+]?\d+)?", answer
                )
            )
            print(num)
            if num != "":
                numerized.append(num)

        if len(numerized) > 0:
            consensus = most_common(numerized)
        else:
            consensus = "NA"

        return consensus



import anthropic
from .base import BaseModel
from typing import Optional


class AnthropicAPIModel(BaseModel):
    def __init__(
        self,
        model: Optional[str] = "claude-3-haiku-20240307",
        temperature: float = 0.0,
        max_tokens: int = 1_000,
    ) -> None:
        super().__init__(model=model, temperature=temperature, max_tokens=max_tokens)

        self.client = anthropic.Anthropic()

    def model_type(self):
        return "text"

    def complete(
        self,
        query: str,
        context: str,
        task_description: Optional[str] = None,
        system_prompt: Optional[str] = None,
    ):
        if task_description:
            user_prompt = query + "\n" + task_description + "\n" + context
        else:
            user_prompt = query + "\n" + context

        message = self.client.messages.create(
            model=self.model,
            max_tokens=self.max_tokens,
            temperature=self.temperature,
            system=system_prompt,
            messages=[
                {
                    "role": "user",
                    "content": [
                        {
                            "type": "text",
                            "text": user_prompt,
                        }
                    ],
                }
            ],
        )
        return message.content[0].text
